@@ -19,8 +19,6 @@
     Home: https://github.com/gorhill/uBlock
 */
 
-/******************************************************************************/
-
 import {
     adminRead,
     browser,
@@ -28,13 +26,9 @@ import {
     localRead, localWrite,
     runtime,
     sessionRead, sessionWrite,
+    windows,
 } from './ext.js';
 
-import {
-    broadcastMessage,
-    ubolLog,
-} from './utils.js';
-
 import {
     defaultRulesetsFromLanguage,
     enableRulesets,
@@ -54,8 +48,13 @@ import {
 } from './mode-manager.js';
 
 import {
-    registerInjectables,
-} from './scripting-manager.js';
+    getMatchedRules,
+    isSideloaded,
+    ubolLog,
+} from './debug.js';
+
+import { broadcastMessage } from './utils.js';
+import { registerInjectables } from './scripting-manager.js';
 
 /******************************************************************************/
 
@@ -253,6 +252,7 @@ function onMessage(request, sender, callback) {
                 hasOmnipotence: results[1],
                 hasGreatPowers: results[2],
                 rulesetDetails: results[3],
+                isSideloaded,
             });
         });
         return true;
@@ -308,6 +308,19 @@ function onMessage(request, sender, callback) {
         });
         return true;
 
+    case 'getMatchedRules':
+        getMatchedRules(request.tabId).then(entries => {
+            callback(entries);
+        });
+        return true;
+
+    case 'showMatchedRules':
+        windows.create({
+            type: 'popup',
+            url: `/matched-rules.html?tab=${request.tabId}`,
+        });
+        break;
+
     default:
         break;
     }
@@ -373,8 +386,16 @@ async function start() {
     }
 }
 
-try {
-    start();
-} catch(reason) {
+// https://github.com/uBlockOrigin/uBOL-home/issues/199
+// Force a restart of the extension once when an "internal error" occurs
+start().then(( ) => {
+    localWrite({ goodStart: true });
+}).catch(reason => {
     console.trace(reason);
-}
+    localRead('goodStart').then((bin = {}) => {
+        if ( bin.goodStart === false ) { return; }
+        localWrite({ goodStart: false }).then(( ) => {
+            runtime.reload();
+        });
+    });
+});
