@@ -19,18 +19,13 @@
     Home: https://github.com/gorhill/uBlock
 */
 
-/* jshint esversion:11 */
-
-'use strict';
-
-/******************************************************************************/
+import * as ut from './utils.js';
 
 import { browser } from './ext.js';
 import { fetchJSON } from './fetch.js';
-import { getFilteringModeDetails } from './mode-manager.js';
 import { getEnabledRulesetsDetails } from './ruleset-manager.js';
-
-import * as ut from './utils.js';
+import { getFilteringModeDetails } from './mode-manager.js';
+import { ubolLog } from './debug.js';
 
 /******************************************************************************/
 
@@ -276,7 +271,7 @@ function registerProcedural(context) {
         allFrames: true,
         matches,
         excludeMatches,
-        runAt: 'document_end',
+        runAt: 'document_start',
     };
 
     // register
@@ -478,6 +473,7 @@ function registerScriptlet(context, scriptletDetails) {
             //   `MAIN` world not yet supported in Firefox
             if ( isGecko === false ) {
                 directive.world = 'MAIN';
+                directive.matchOriginAsFallback = true;
             }
 
             // register
@@ -542,13 +538,13 @@ async function registerInjectables(origins) {
     toRemove.push(...Array.from(before.keys()));
 
     if ( toRemove.length !== 0 ) {
-        ut.ubolLog(`Unregistered ${toRemove} content (css/js)`);
+        ubolLog(`Unregistered ${toRemove} content (css/js)`);
         await browser.scripting.unregisterContentScripts({ ids: toRemove })
             .catch(reason => { console.info(reason); });
     }
 
     if ( toAdd.length !== 0 ) {
-        ut.ubolLog(`Registered ${toAdd.map(v => v.id)} content (css/js)`);
+        ubolLog(`Registered ${toAdd.map(v => v.id)} content (css/js)`);
         await browser.scripting.registerContentScripts(toAdd)
             .catch(reason => { console.info(reason); });
     }
