@@ -19,19 +19,17 @@
     Home: https://github.com/gorhill/uBlock
 */
 
-'use strict';
+import * as makeScriptlet from './make-scriptlets.js';
+import * as sfp from './js/static-filtering-parser.js';
 
-/******************************************************************************/
+import { createHash, randomBytes } from 'crypto';
 
+import { dnrRulesetFromRawLists } from './js/static-dnr-filtering.js';
 import fs from 'fs/promises';
 import https from 'https';
 import path from 'path';
 import process from 'process';
-import { createHash, randomBytes } from 'crypto';
 import redirectResourcesMap from './js/redirect-resources.js';
-import { dnrRulesetFromRawLists } from './js/static-dnr-filtering.js';
-import * as sfp from './js/static-filtering-parser.js';
-import * as makeScriptlet from './make-scriptlets.js';
 import { safeReplace } from './safe-replace.js';
 
 /******************************************************************************/
@@ -106,8 +104,7 @@ const log = (text, silent = false) => {
 const urlToFileName = url => {
     return url
         .replace(/^https?:\/\//, '')
-        .replace(/\//g, '_')
-        ;
+        .replace(/\//g, '_');
 };
 
 const fetchText = (url, cacheDir) => {
@@ -338,24 +335,6 @@ async function processNetworkFilters(assetDetails, network) {
     log(`\tRejected filter count: ${network.rejectedFilterCount}`);
     log(`Output rule count: ${rules.length}`);
 
-    // https://developer.mozilla.org/en-US/docs/Mozilla/Add-ons/WebExtensions/API/declarativeNetRequest/RuleCondition#browser_compatibility
-    // isUrlFilterCaseSensitive is true by default in Chromium. It will be
-    // false by default in Chromium 118+.
-    if ( platform !== 'firefox' ) {
-        for ( const rule of rules ) {
-            const { condition } = rule;
-            if ( condition === undefined ) { continue; }
-            if ( condition.urlFilter === undefined ) {
-                if ( condition.regexFilter === undefined ) { continue; }
-            }
-            if ( condition.isUrlFilterCaseSensitive === undefined ) {
-                condition.isUrlFilterCaseSensitive = false;
-            } else if ( condition.isUrlFilterCaseSensitive === true ) {
-                condition.isUrlFilterCaseSensitive = undefined;
-            }
-        }
-    }
-
     // Minimize requestDomains arrays
     for ( const rule of rules ) {
         const condition = rule.condition;
@@ -383,8 +362,7 @@ async function processNetworkFilters(assetDetails, network) {
     log(plainGood
         .filter(rule => Array.isArray(rule._warning))
         .map(rule => rule._warning.map(v => `\t\t${v}`))
-        .join('\n'),
-        true
+        .join('\n'), true
     );
 
     const regexes = rules.filter(rule => isGood(rule) && isRegex(rule));
@@ -1296,7 +1274,7 @@ async function main() {
     // Handpicked rulesets from abroad
     await rulesetFromURLs({
         id: 'stevenblack-hosts',
-        name: 'Steven Black\'s hosts file',
+        name: 'Steven Black’s Unified Hosts (adware + malware)',
         enabled: false,
         urls: [ 'https://raw.githubusercontent.com/StevenBlack/hosts/master/hosts' ],
         homeURL: 'https://github.com/StevenBlack/hosts#readme',
