@@ -19,17 +19,13 @@
     Home: https://github.com/gorhill/uBlock
 */
 
-'use strict';
-
 /******************************************************************************/
 
-import contextMenu from './contextmenu.js';
-import logger from './logger.js';
-import staticNetFilteringEngine from './static-net-filtering.js';
-import µb from './background.js';
-import webext from './webext.js';
-import { orphanizeString } from './text-utils.js';
-import { redirectEngine } from './redirect-engine.js';
+import {
+    domainFromHostname,
+    hostnameFromURI,
+    isNetworkURI,
+} from './uri-utils.js';
 
 import {
     sessionFirewall,
@@ -37,11 +33,13 @@ import {
     sessionURLFiltering,
 } from './filtering-engines.js';
 
-import {
-    domainFromHostname,
-    hostnameFromURI,
-    isNetworkURI,
-} from './uri-utils.js';
+import contextMenu from './contextmenu.js';
+import logger from './logger.js';
+import { orphanizeString } from './text-utils.js';
+import { redirectEngine } from './redirect-engine.js';
+import staticNetFilteringEngine from './static-net-filtering.js';
+import webext from './webext.js';
+import µb from './background.js';
 
 /*******************************************************************************
 
@@ -55,6 +53,9 @@ To create a log of net requests
 /******************************************************************************/
 
 const NetFilteringResultCache = class {
+    shelfLife = 15000;
+    extensionOriginURL = vAPI.getURL('/');
+
     constructor() {
         this.pruneTimer = vAPI.defer.create(( ) => {
             this.prune();
@@ -174,9 +175,6 @@ const NetFilteringResultCache = class {
     }
 };
 
-NetFilteringResultCache.prototype.shelfLife = 15000;
-NetFilteringResultCache.prototype.extensionOriginURL = vAPI.getURL('/');
-
 /******************************************************************************/
 
 // Frame stores are used solely to associate a URL with a frame id.
@@ -276,18 +274,16 @@ const FrameStore = class {
     }
 
     static factory(frameURL, parentId = -1) {
-        const entry = FrameStore.junkyard.pop();
-        if ( entry === undefined ) {
-            return new FrameStore(frameURL, parentId);
+        const FS = FrameStore;
+        if ( FS.junkyard.length !== 0 ) {
+            return FS.junkyard.pop().init(frameURL, parentId);
         }
-        return entry.init(frameURL, parentId);
+        return new FS(frameURL, parentId);
     }
+    static junkyard = [];
+    static junkyardMax = 50;
 };
 
-// To mitigate memory churning
-FrameStore.junkyard = [];
-FrameStore.junkyardMax = 50;
-
 /******************************************************************************/
 
 const CountDetails = class {
@@ -314,19 +310,26 @@ const HostnameDetails = class {
     }
     init(hostname) {
         this.hostname = hostname;
+        this.cname = vAPI.net.canonicalNameFromHostname(hostname);
         this.counts.reset();
+        return this;
     }
     dispose() {
-        this.hostname = '';
-        if ( HostnameDetails.junkyard.length < HostnameDetails.junkyardMax ) {
-            HostnameDetails.junkyard.push(this);
-        }
+        const HD = HostnameDetails;
+        if ( HD.junkyard.length >= HD.junkyardMax ) { return; }
+        HD.junkyard.push(this);
     }
+    static factory(hostname) {
+        const HD = HostnameDetails;
+        if ( HD.junkyard.length !== 0 ) {
+            return HD.junkyard.pop().init(hostname);
+        }
+        return new HD(hostname);
+    }
+    static junkyard = [];
+    static junkyardMax = 100;
 };
 
-HostnameDetails.junkyard = [];
-HostnameDetails.junkyardMax = 100;
-
 const HostnameDetailsMap = class extends Map {
     reset() {
         this.clear();
@@ -379,11 +382,13 @@ const PageStore = class {
 
         // If we are navigating from-to same site, remember whether large
         // media elements were temporarily allowed.
-        if (
-            typeof this.allowLargeMediaElementsUntil !== 'number' ||
-            tabContext.rootHostname !== this.tabHostname
-        ) {
-            this.allowLargeMediaElementsUntil = Date.now();
+        const now = Date.now();
+        if ( typeof this.allowLargeMediaElementsUntil !== 'number' ) {
+            this.allowLargeMediaElementsUntil = now;
+        } else if ( tabContext.rootHostname !== this.tabHostname ) {
+            if ( this.tabHostname.endsWith('about-scheme') === false ) {
+                this.allowLargeMediaElementsUntil = now;
+            }
         }
 
         this.tabHostname = tabContext.rootHostname;
@@ -622,7 +627,7 @@ const PageStore = class {
         ) {
             this.hostnameDetailsMap.set(
                 this.tabHostname,
-                new HostnameDetails(this.tabHostname)
+                HostnameDetails.factory(this.tabHostname)
             );
         }
         return this.hostnameDetailsMap;
@@ -700,7 +705,7 @@ const PageStore = class {
             const hostname = journal[i+0];
             let hnDetails = this.hostnameDetailsMap.get(hostname);
             if ( hnDetails === undefined ) {
-                hnDetails = new HostnameDetails(hostname);
+                hnDetails = HostnameDetails.factory(hostname);
                 this.hostnameDetailsMap.set(hostname, hnDetails);
                 this.contentLastModified = now;
             }
@@ -739,10 +744,8 @@ const PageStore = class {
                 aggregateAllowed += 1;
             }
         }
-        if ( aggregateAllowed !== 0 || aggregateBlocked !== 0 ) {
-            µb.localSettings.blockedRequestCount += aggregateBlocked;
-            µb.localSettings.allowedRequestCount += aggregateAllowed;
-            µb.localSettingsLastModified = now;
+        if ( aggregateAllowed || aggregateBlocked ) {
+            µb.incrementRequestStats(aggregateBlocked, aggregateAllowed);
         }
         journal.length = 0;
     }
@@ -934,19 +937,14 @@ const PageStore = class {
     }
 
     redirectNonBlockedRequest(fctxt) {
-        const transformDirectives = staticNetFilteringEngine.transformRequest(fctxt);
-        const pruneDirectives = fctxt.redirectURL === undefined &&
-            staticNetFilteringEngine.hasQuery(fctxt) &&
-            staticNetFilteringEngine.filterQuery(fctxt) ||
-            undefined;
-        if ( transformDirectives === undefined && pruneDirectives === undefined ) { return; }
+        const directives = [];
+        staticNetFilteringEngine.transformRequest(fctxt, directives);
+        if ( staticNetFilteringEngine.hasQuery(fctxt) ) {
+            staticNetFilteringEngine.filterQuery(fctxt, directives);
+        }
+        if ( directives.length === 0 ) { return; }
         if ( logger.enabled !== true ) { return; }
-        if ( transformDirectives !== undefined ) {
-            fctxt.pushFilters(transformDirectives.map(a => a.logData()));
-        }
-        if ( pruneDirectives !== undefined ) {
-            fctxt.pushFilters(pruneDirectives.map(a => a.logData()));
-        }
+        fctxt.pushFilters(directives.map(a => a.logData()));
         if ( fctxt.redirectURL === undefined ) { return; }
         fctxt.pushFilter({
             source: 'redirect',
@@ -954,6 +952,19 @@ const PageStore = class {
         });
     }
 
+    skipMainDocument(fctxt) {
+        const directives = staticNetFilteringEngine.urlSkip(fctxt);
+        if ( directives === undefined ) { return; }
+        if ( logger.enabled !== true ) { return; }
+        fctxt.pushFilters(directives.map(a => a.logData()));
+        if ( fctxt.redirectURL !== undefined ) {
+            fctxt.pushFilter({
+                source: 'redirect',
+                raw: fctxt.redirectURL
+            });
+        }
+    }
+
     filterCSPReport(fctxt) {
         if (
             sessionSwitches.evaluateZ(
